/*
 * Example for WiFi 7 click
 *
 * Date         Apr 2017.
 * Author       Milos V.
 *
 * Test configuration PIC32 :
 *  MCU           :	P32MX795F512L
 *  Dev. Board    :	EasyPIC Fusion v7
 *  SW            :	mikroC PRO for PIC32 v3.6.1
 *
 * Select example by editing demo_config.h.
 * Include example.pld in your project
 *
 ******************************************************************************/
#include "winc1500_api.h"   // primary WINC1500 include file
#include "demo_config.h"    // selects which demo to run

sbit WIFI_7_RST at LATC3_bit;
sbit WIFI_7_EN  at LATD1_bit;
sbit WIFI_7_CS  at LATC4_bit;

static void systemInit( void )
{
    TRISC3_bit = 0;
    TRISD1_bit = 0;
    TRISC4_bit = 0;

    UART5_Init( 115200 );

    SPI3_Init_Advanced( _SPI_MASTER, _SPI_8_BIT, 16, _SPI_SS_DISABLE,
                        _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW,
                        _SPI_IDLE_2_ACTIVE );

    AD1PCFG     = 0xFFFF;              // Turning all analog pins to digital
    INT2IP0_bit = 1;                   // Set INT1 interrupt
    INT2IP1_bit = 1;                   // Set interrupt priorities
    INT2IP2_bit = 1;                   // Set inrrupt priority to 4
    INT2EP_bit  = 1;			// Rising edge
    INT2IE_bit  = 1;
    T1CON	 = 0x8010;
    T1IP0_bit	 = 1;
    T1IP1_bit	 = 1;
    T1IP2_bit	 = 1;
    T1IF_bit	 = 0;
    T1IE_bit	 = 1;
    PR1		 = 10000;
    TMR1        = 0;
    EnableInterrupts();
}

void m2mStub_EintEnable( void )
{
    INT2IE_bit = 1;
}

void m2mStub_EintDisable( void )
{
    INT2IE_bit = 0;
}

void main(void)
{
    systemInit();

#if defined(USING_PICTAIL)
    printf("Project built for PICtail\r\n");
#elif defined(USING_CLICK_BOARD)
    printf("Project built for Click Board\r\n");
#endif

    printf("Starting driver initialization...\r\n");
    hal_wifi7_init();
    m2m_wifi_init();

    while (1)
    {
        ApplicationTask();
        m2m_wifi_task();
    }
}

void ext_ISR() iv IVT_EXTERNAL_2 ilevel 7 ics ICS_SRS
{
    m2m_EintHandler();
    INT2IF_bit = 0;
}

void Timer1Interrupt() iv IVT_TIMER_1 ilevel 7 ics ICS_SRS 
{
    T1IF_bit	 = 0;
    hal_wifi7_tick();
}

/************************************************************ END OF FILE *****/
