#ifndef WIFI_7_H
#define WIFI_7_H

#include <stdint.h>
#include <stdlib.h>
#include <stdbool.h>

#ifdef HOST_MCU_BIG_ENDIAN
    #define FIX_ENDIAN_32(x) ((((uint32_t)(x) & 0xff000000) >> 24) |    \
                              (((uint32_t)(x) & 0x00ff0000) >> 8)  |    \
                              (((uint32_t)(x) & 0x0000ff00) << 8)  |    \
                              (((uint32_t)(x) & 0x000000ff) << 24))

    #define FIX_ENDIAN_16(x) ((((uint16_t)(x) & 0xff00) >> 8) |     \
                              (((uint16_t)(x) & 0x00ff) << 8))
#else
    #define FIX_ENDIAN_32(x)  (x)
    #define FIX_ENDIAN_16(x)  (x)
#endif

#ifdef HOST_MCU_BIG_ENDIAN
    #define _htonl(x)   (x)
    #define _htons(x)   (x)
#else
    #define _htonl(x) ((((uint32_t)(x) & 0xff000000) >> 24) |    \
                       (((uint32_t)(x) & 0x00ff0000) >> 8)  |    \
                       (((uint32_t)(x) & 0x0000ff00) << 8)  |    \
                       (((uint32_t)(x) & 0x000000ff) << 24))

    #define _htons(x) ((((uint16_t)(x) & 0xff00) >> 8) |     \
                       (((uint16_t)(x) & 0x00ff) << 8))
#endif

#define _ntohl              _htonl
#define _ntohs              _htons


#define BSP_MIN(x,y) ((x)>(y)?(y):(x))

#define M2M_ERR_MEM_ALLOC       ((int8_t)-3)

#define NBIT31                (0x80000000)
#define NBIT30                (0x40000000)
#define NBIT29                (0x20000000)
#define NBIT28                (0x10000000)
#define NBIT27                (0x08000000)
#define NBIT26                (0x04000000)
#define NBIT25                (0x02000000)
#define NBIT24                (0x01000000)
#define NBIT23                (0x00800000)
#define NBIT22                (0x00400000)
#define NBIT21                (0x00200000)
#define NBIT20                (0x00100000)
#define NBIT19                (0x00080000)
#define NBIT18                (0x00040000)
#define NBIT17                (0x00020000)
#define NBIT16                (0x00010000)
#define NBIT15                (0x00008000)
#define NBIT14                (0x00004000)
#define NBIT13                (0x00002000)
#define NBIT12                (0x00001000)
#define NBIT11                (0x00000800)
#define NBIT10                (0x00000400)
#define NBIT9                 (0x00000200)
#define NBIT8                 (0x00000100)
#define NBIT7                 (0x00000080)
#define NBIT6                 (0x00000040)
#define NBIT5                 (0x00000020)
#define NBIT4                 (0x00000010)
#define NBIT3                 (0x00000008)
#define NBIT2                 (0x00000004)
#define NBIT1                 (0x00000002)
#define NBIT0                 (0x00000001)

#define NMI_PERIPH_REG_BASE     0x1000
#define CHIP_ID_REG             (NMI_PERIPH_REG_BASE)
#define RF_REV_ID_REG           (0x13f4)
#define rNMI_GP_REG_0           (0x149c)
#define rNMI_GP_REG_1           (0x14A0)
#define rNMI_GP_REG_2           (0xc0008)
#define rNMI_GLB_RESET          (0x1400)
#define rNMI_BOOT_RESET_MUX     (0x1118)
#define NMI_STATE_REG           (0x108c)
#define BOOTROM_REG             (0xc000c)
#define NMI_REV_REG             (0x207ac)    /*Also, Used to load ATE firmware from SPI Flash and to ensure that it is running too*/
#define NMI_REV_REG_ATE         (0x1048)     /*Revision info register in case of ATE FW*/
#define M2M_WAIT_FOR_HOST_REG   (0x207bc)
#define M2M_FINISH_INIT_STATE   0x02532636UL
#define M2M_FINISH_BOOT_ROM     0x10add09eUL
#define M2M_START_FIRMWARE      0xef522f61UL
#define M2M_START_PS_FIRMWARE   0x94992610UL
#define M2M_ATE_FW_START_VALUE  (0x3C1CD57D)    /*Also, Change this value in boot_firmware if it will be changed here*/
#define M2M_ATE_FW_IS_UP_VALUE  (0xD75DC1C3)    /*Also, Change this value in ATE (Burst) firmware if it will be changed here*/

#define REV_2B0                 (0x2B0)
#define REV_B0                  (0x2B0)
#define REV_3A0                 (0x3A0)
#define GET_CHIPID()            GetChipId()
#define ISNMC1000(id)           (((id & 0xfffff000) == 0x100000) ? 1 : 0)
#define ISNMC1500(id)           (((id & 0xfffff000) == 0x150000) ? 1 : 0)
#define ISNMC3000(id)           ((((id) & 0xfff00000) == 0x300000) ? 1 : 0)
#define REV(id)                 (((id) & 0x00000fff))
#define EFUSED_MAC(value)       (value & 0xffff0000)

#define rHAVE_SDIO_IRQ_GPIO_BIT     (NBIT0)
#define rHAVE_USE_PMU_BIT           (NBIT1)
#define rHAVE_SLEEP_CLK_SRC_RTC_BIT (NBIT2)
#define rHAVE_SLEEP_CLK_SRC_XO_BIT  (NBIT3)
#define rHAVE_EXT_PA_INV_TX_RX      (NBIT4)
#define rHAVE_LEGACY_RF_SETTINGS    (NBIT5)
#define rHAVE_LOGS_DISABLED_BIT     (NBIT6)
#define rHAVE_ETHERNET_MODE_BIT     (NBIT7)
#define rHAVE_RESERVED1_BIT         (NBIT8)

//==============================================================================
// CONSTANTS
//==============================================================================
#define  AF_INET            2    // The AF_INET is the address family used for IPv4 and
                                 //  is the only supported type in the WINC1500 driver.
#define  SOCK_STREAM        1    // The IPv4 supported socket type for reliable connection-oriented
                                 //  stream connection (TCP)
#define  SOCK_DGRAM         2    // The IPv4 supported socket type for an unreliable
                                 //  connectionless datagram connection (UDP)
#define SOCKET_FLAGS_SSL    0x01 // Bit mask for the flags parameter when creating an SSL socket
#define TCP_SOCK_MAX        7    // Maximum number of simultaneous TCP sockets
#define UDP_SOCK_MAX        4    // Maximum number of simultaneous UDP sockets
#define MAX_SOCKET         (TCP_SOCK_MAX + UDP_SOCK_MAX) // Maximum number of all socket types

#define SOL_SOCKET          1    // parameter for 'level' field in setsockopt and getsockopt
#define SOL_SSL_SOCKET      2    // SSL socket option

#define SOCKET_BUFFER_MAX_LENGTH  1400  // Maximum allowed size for a socket data buffer.
                                        //  Used with sendsock() to ensure that the buffer
                                        //  sent is within the allowed range.

#define M2M_HOSTNAME_MAX_SIZE        64      // Maximum allowed size for a host domain name
#define M2M_INET4_ADDRSTRLEN         (sizeof("255.255.255.255"))  // Max size of ipV4 string



#define MAJOR_SHIFT                 (8)
#define MINOR_SHIFT                 (4)
#define PATCH_SHIFT                 (0)

#define DRIVER_VERSION_SHIFT        (16)
#define FIRMWARE_VERSION_SHIFT      (0)

//==============================================================================
// MACROS
//==============================================================================
#define GET_MAJOR(ver_info_hword)           ((uint8_t)((ver_info_hword) >> MAJOR_SHIFT) & 0xff)
#define GET_MINOR(ver_info_hword)           ((uint8_t)((ver_info_hword) >> MINOR_SHIFT) & 0x0f)
#define GET_PATCH(ver_info_hword)           ((uint8_t)((ver_info_hword) >> PATCH_SHIFT) & 0x0f)

#define GET_FIRMWARE_VERSION(ver_info_word) ((uint16_t) ((ver_info_word) >> FIRMWARE_VERSION_SHIFT))
#define GET_DRIVER_VERSION(ver_info_word)   ((uint16_t) ((ver_info_word) >> DRIVER_VERSION_SHIFT))

#define GET_DRIVER_MAJOR(ver_info_word)     GET_MAJOR(GET_DRIVER_VERSION(ver_info_word))
#define GET_DRIVER_MINOR(ver_info_word)     GET_MINOR(GET_DRIVER_VERSION(ver_info_word))
#define GET_DRIVER_PATCH(ver_info_word)     GET_PATCH(GET_DRIVER_VERSION(ver_info_word))

#define GET_FIRMWARE_MAJOR(ver_info_word)   GET_MAJOR(GET_FIRMWARE_VERSION(ver_info_word))
#define GET_FIRMWARE_MINOR(ver_info_word)   GET_MINOR(GET_FIRMWARE_VERSION(ver_info_word))
#define GET_FIRMWARE_PATCH(ver_info_word)   GET_PATCH(GET_FIRMWARE_VERSION(ver_info_word))

#define MAKE_VERSION(major, minor, patch) ( \
    ((uint16_t)((major)  & 0xff)  << MAJOR_SHIFT) | \
    ((uint16_t)((minor)  & 0x0f)  << MINOR_SHIFT) | \
    ((uint16_t)((patch)  & 0x0f)  << PATCH_SHIFT))

#define MAKE_VERSION_INFO(fw_major, fw_minor, fw_patch, drv_major, drv_minor, drv_patch)          \
    (                                                                                              \
    ( ((uint32_t)MAKE_VERSION((fw_major),  (fw_minor),  (fw_patch)))  << FIRMWARE_VERSION_SHIFT) | \
    ( ((uint32_t)MAKE_VERSION((drv_major), (drv_minor), (drv_patch))) << DRIVER_VERSION_SHIFT))

// Base values for Wi-Fi Command groups
#define CONFIG_CMD_BASE             1       // Base value of all the host configuration commands opcodes
#define STA_CMD_BASE                40      // Base value of all the station mode host commands opcodes
#define AP_CMD_BASE                 70      // Base value of all the Access Point mode host commands opcodes
#define P2P_CMD_BASE                90      // Base value of all the P2P mode host commands opcodes
#define SERVER_CMD_BASE             100     // [not currently supported] Base value of all the power save mode host commands codes

// OTA Definitions
#define OTA_STATUS_VALID            (0x12526285)        // Magic value updated in the Control structure in case of ROLLACK image valid
#define OTA_STATUS_INVALID          (0x23987718)        // Magic value updated in the Control structure in case of ROLLACK image invalid
#define OTA_MAGIC_VALUE             (0x1ABCDEF9)        // Magic value set at the beginning of the OTA image header
#define OTA_FORMAT_VER_0            (0)                 // Until 19.2.2 format
#define OTA_FORMAT_VER_1            (1)                 // Starting from 19.3.0 CRC is used and sequence number is used
#define OTA_SHA256_DIGEST_SIZE      (32)                // Sha256 digest size in the OTA image, the sha256
                                                        //  digest is set at the beginning of image before the OTA header
#define M2M_MAGIC_APP               (0xef522f61UL)      // Magic value set at the beginning of the Cortus OTA image header

// Base value for OTA Command group
//  The OTA have a special group so can extended from 1-MAX_GRP_NUM_REQ
#define M2M_OTA_CMD_BASE            100

// Base value for Crypto Command group
//  The crypto have a special group so can extended from 1-MAX_GRP_NUM_REQ
#define M2M_CRYPTO_CMD_BASE                                    1


#define MAX_GRP_NUM_REQ             (127)   // max number of request in one group equal to 127
                                            //  as the last bit reserved for config or data pkt

#define M2M_ETHERNET_HDR_OFFSET         34      // Offset of the Ethernet header within the WLAN Tx Buffer
#define M2M_ETHERNET_HDR_LEN            14      // Length of the Etherenet header in bytes

#define M2M_BUFFER_MAX_SIZE      (1600UL - 4)    // Maximum size for the shared packet buffer
#define M2M_MAC_ADDRES_LEN             6

#define REL_19_5_2_VER            MAKE_VERSION_INFO(19,5,2,19,3,0)
#define REL_19_5_1_VER            MAKE_VERSION_INFO(19,5,1,19,3,0)
#define REL_19_5_0_VER            MAKE_VERSION_INFO(19,5,0,19,3,0)
#define REL_19_4_6_VER            MAKE_VERSION_INFO(19,4,6,19,3,0)
#define REL_19_4_5_VER            MAKE_VERSION_INFO(19,4,5,19,3,0)
#define REL_19_4_4_VER            MAKE_VERSION_INFO(19,4,4,19,3,0)
#define REL_19_4_3_VER            MAKE_VERSION_INFO(19,4,3,19,3,0)
#define REL_19_4_2_VER            MAKE_VERSION_INFO(19,4,2,19,3,0)
#define REL_19_4_1_VER            MAKE_VERSION_INFO(19,4,1,19,3,0)
#define REL_19_4_0_VER            MAKE_VERSION_INFO(19,4,0,19,3,0)
#define REL_19_3_1_VER            MAKE_VERSION_INFO(19,3,1,19,3,0)
#define REL_19_3_0_VER            MAKE_VERSION_INFO(19,3,0,19,3,0)
#define REL_19_2_2_VER            MAKE_VERSION_INFO(19,2,2,19,2,0)
#define REL_19_2_1_VER            MAKE_VERSION_INFO(19,2,1,19,2,0)
#define REL_19_2_0_VER            MAKE_VERSION_INFO(19,2,0,19,2,0)
#define REL_19_1_0_VER            MAKE_VERSION_INFO(19,1,0,18,2,0)
#define REL_19_0_0_VER            MAKE_VERSION_INFO(19,0,0,18,1,1)

//------------------------
// Firmware version number
//------------------------
#define FIRMWARE_RELEASE_VERSION_MAJOR_NO   (19)
#define FIRMWARE_RELEASE_VERSION_MINOR_NO   (5)
#define FIRMWARE_RELEASE_VERSION_PATCH_NO   (2)


#if !defined(FIRMWARE_RELEASE_VERSION_MAJOR_NO) || !defined(FIRMWARE_RELEASE_VERSION_MINOR_NO)
#error Undefined version number
#endif



#define M2M_MAX_SSID_LEN            33      // Maximum size for the Wi-Fi SSID including the NULL termination
#define M2M_WIFI_MIN_PSK_LEN        9       // Minimum size for the WPA PSK including the NULL termination
#define M2M_MAX_PSK_LEN             65      // Maximum size for the WPA PSK including the NULL termination
#define M2M_DEVICE_NAME_MAX         48      // Maximum Size for the device name including the NULL termination
#define M2M_WIFI_MAX_HIDDEN_SITES   4       // Max number of hidden SSID's supported by scan request
#define M2M_1X_USR_NAME_MAX         21      // The maximum size of the user name including the NULL termination.
                                            //  It is used for RADIUS authentication in case of connecting the device to
                                            //  an AP secured with WPA-Enterprise.
#define M2M_WIFI_PASSWORD_1X_MAX    41      // The maximum size of the password including the NULL termination.
                                            //  It is used for RADIUS authentication in case of connecting the device to
                                            //  an AP secured with WPA-Enterprise.
#define M2M_WIFI_CUST_IE_LEN_MAX    252     // Maximum size of IE (Information Element)

#define M2M_WIFI_PWR_DEFAULT        PWR_HIGH

#define M2M_WIFI_WEP_40_KEY_STRING_SIZE      ((uint8_t)10)   // WEP key size in bytes for 40 bit string passphrase
#define WEP_104_KEY_STRING_SIZE              ((uint8_t)26)   // WEP key size in bytes for 104 bit string passphrase
#define M2M_WIFI_WEP_KEY_MAX_INDEX           ((uint8_t)4)    // Max key index value for WEP authentication
#define M2M_WIFI_SHA256_CONTEXT_BUF_LEN      (128)           // sha256 context size
#define M2M_WIFI_SCAN_DEFAULT_NUM_SLOTS      (2)             // Default number of scan slots performed by the WINC board
#define M2M_WIFI_SCAN_DEFAULT_SLOT_TIME      (30)            // Default duration in miliseconds of a scan slots performed by the WINC board
#define M2M_WIFI_SCAN_DEFAULT_NUM_PROBE      (2)             // Default number of scan slots performed by the WINC board

//----------------
// SSL Definitions
//----------------

#define M2M_WIFI_TLS_CRL_DATA_MAX_LEN      64  // Maximum data length in a CRL entry (= Hash length for SHA512)
#define M2M_WIFI_TLS_CRL_MAX_ENTRIES       10  // Maximum number of entries in a CRL


//----------------
// TLS Definitions
//----------------
// Maximum length for each TLS certificate file name including null terminator.
#define TLS_FILE_NAME_MAX								48

// Maximum number of certificates allowed in TLS_SRV section.
#define TLS_SRV_SEC_MAX_FILES							8

// Length of certificate struct start pattern.
#define TLS_SRV_SEC_START_PATTERN_LEN					8


#define FLASH_MAP_VER_0        (0)
#define FLAAH_MAP_VER_1        (1)

#define FLASH_MAP_VERSION    FLAAH_MAP_VER_1

//#define DOWNLOAD_ROLLBACK
//#define OTA_GEN
#define _PROGRAM_POWER_SAVE_

/* =======*=======*=======*=======*=======
 *       General Sizes for Flash Memory
 * =======*=======*=======*=======*=======
 */

#define FLASH_START_ADDR                    (0UL)
/*!<Starting Address of Flash Memory
 *
 */
#define FLASH_BLOCK_SIZE                    (32 * 1024UL)
/*!<Block Size in Flash Memory
 */
#define FLASH_SECTOR_SZ                     (4 * 1024UL)
/*!<Sector Size in Flash Memory
 */
#define FLASH_PAGE_SZ                       (256)
/*!<Page Size in Flash Memory
 */
#define FLASH_2M_TOTAL_SZ                   (256 * 1024UL)
/*!<Total Size of 2M Flash Memory
 */
#define FLASH_4M_TOTAL_SZ                   (512 * 1024UL)
/*!<Total Size of 4M Flash Memory
 */
#define FLASH_8M_TOTAL_SZ                   (1024 * 1024UL)
/*!<Total Size of 8M Flash Memory
 */

/*
 * Detailed Sizes and locations for Flash Memory:
 *  _______________________________ ___________________________ _______________________________________________
 * | Starting Address   |    Size    |    Location's Name       |       Description                                       |
 * |____________________|___________|___________________________|_______________________________________________|
 * |      0K            |     4K    |    Boot Firmware          |    Firmware to select which version to run        |
 * |      4K            |     8K    |    Control Section        |    Structured data used by Boot firmware        |
 * |     12K            |     4K    |    PLL+GAIN :             |    LookUp Table for PLL and Gain calculations    |
 * |                    |           |    PLL  Size = 1K         |        PLL                                         |
 * |                    |           |    GAIN Size = 3K         |        Gain configuration                         |
 * |     16K            |     4K    |    CERTIFICATE            |    X.509 Certificate storage                    |
 * |     20K            |     4K    |    Scratch Section        |    Empty Section                                |
 * |     24K            |     4K    |    Reserved TLS Server    |    Reserved                                    |
 * |     28K            |     8K    |    HTTP Files             |    Files used with Provisioning Mode            |
 * |     36K            |     4K    |    Connection Parameters  |    Parameters for success connection to AP        |
 * |     40K            |   236K    |    Main Firmware/program  |    Main Firmware to run WiFi Chip                |
 * |    276K            |   236K    |    OTA Firmware           |    OTA firmware                                |
 * |    512 K                                                       Total flash size                            |
 * |____________________|___________|___________________________|_______________________________________________|
 *
 *
 * *Keys for Comments with each MACRO:
 *         "L:xxxK" -means-> location   :xxxK
 *        "S:xxxK" -means-> Size is     :xxxK
 */

/*
 * Boot Firmware: which used to select which firmware to run
 *
 */
#define M2M_BOOT_FIRMWARE_STARTING_ADDR     (FLASH_START_ADDR)
#define M2M_BOOT_FIRMWARE_FLASH_SZ          (FLASH_SECTOR_SZ)

/*
 * Control Section: which used by Boot firmware
 *
 */
#define M2M_CONTROL_FLASH_OFFSET            (M2M_BOOT_FIRMWARE_STARTING_ADDR + M2M_BOOT_FIRMWARE_FLASH_SZ)
#define M2M_CONTROL_FLASH_BKP_OFFSET        (M2M_CONTROL_FLASH_OFFSET + FLASH_SECTOR_SZ)
#define M2M_CONTROL_FLASH_SEC_SZ            (FLASH_SECTOR_SZ)
#define M2M_CONTROL_FLASH_TOTAL_SZ          (FLASH_SECTOR_SZ * 2)

/*
 * LUT for PLL and TX Gain settings:
 *
 */
#define M2M_PLL_FLASH_OFFSET                (M2M_CONTROL_FLASH_OFFSET + M2M_CONTROL_FLASH_TOTAL_SZ)
#define M2M_PLL_FLASH_SZ                    (1024 * 1)
#define M2M_GAIN_FLASH_OFFSET               (M2M_PLL_FLASH_OFFSET + M2M_PLL_FLASH_SZ)
#define M2M_GAIN_FLASH_SZ                   (M2M_CONFIG_SECT_TOTAL_SZ - M2M_PLL_FLASH_SZ)
#define M2M_CONFIG_SECT_TOTAL_SZ            (FLASH_SECTOR_SZ)

/*
 * Certificate:
 *
 */
#define M2M_TLS_FLASH_ROOTCERT_CACHE_OFFSET (M2M_PLL_FLASH_OFFSET + M2M_CONFIG_SECT_TOTAL_SZ)
#define M2M_TLS_FLASH_ROOTCERT_CACHE_SIZE   (FLASH_SECTOR_SZ * 1)

/*
 * Scratch:
 *
 */
#define M2M_TLS_FLASH_SESSION_CACHE_OFFSET  (M2M_TLS_FLASH_ROOTCERT_CACHE_OFFSET + M2M_TLS_FLASH_ROOTCERT_CACHE_SIZE)
#define M2M_TLS_FLASH_SESSION_CACHE_SIZE    (FLASH_SECTOR_SZ * 1)

/*
 * reserved section
 *
 */
#define M2M_RESERVED_FLASH_OFFSET           (M2M_TLS_FLASH_SESSION_CACHE_OFFSET + M2M_TLS_FLASH_SESSION_CACHE_SIZE)
#define M2M_RESERVED_FLASH_SZ               (FLASH_SECTOR_SZ * 1)
/*
 * HTTP Files
 *
 */
#define M2M_HTTP_MEM_FLASH_OFFSET           (M2M_RESERVED_FLASH_OFFSET + M2M_RESERVED_FLASH_SZ)
#define M2M_HTTP_MEM_FLASH_SZ               (FLASH_SECTOR_SZ * 2)
/*
 * Saved Connection Parameters:
 *
 */
#define M2M_CACHED_CONNS_FLASH_OFFSET       (M2M_HTTP_MEM_FLASH_OFFSET + M2M_HTTP_MEM_FLASH_SZ)
#define M2M_CACHED_CONNS_FLASH_SZ           (FLASH_SECTOR_SZ * 1)

/*
 *
 * Common section size
 */

#define M2M_COMMON_DATA_SEC                 (M2M_BOOT_FIRMWARE_FLASH_SZ + M2M_CONTROL_FLASH_TOTAL_SZ + M2M_CONFIG_SECT_TOTAL_SZ + \
                                            M2M_TLS_FLASH_ROOTCERT_CACHE_SIZE + M2M_TLS_FLASH_SESSION_CACHE_SIZE + \
                                            M2M_HTTP_MEM_FLASH_SZ  + M2M_CACHED_CONNS_FLASH_SZ + M2M_RESERVED_FLASH_SZ)
/*
 *
 * OTA image1 Offset
 */

#define M2M_OTA_IMAGE1_OFFSET               (M2M_CACHED_CONNS_FLASH_OFFSET + M2M_CACHED_CONNS_FLASH_SZ)
/*
 * Firmware Offset
 *
 */
#if (defined _FIRMWARE_)||(defined OTA_GEN)
#define M2M_FIRMWARE_FLASH_OFFSET           (0UL)
#else
#if (defined DOWNLOAD_ROLLBACK)
#define M2M_FIRMWARE_FLASH_OFFSET           (M2M_OTA_IMAGE2_OFFSET)
#else
#define M2M_FIRMWARE_FLASH_OFFSET           (M2M_OTA_IMAGE1_OFFSET)
#endif
#endif
/*
 *
 * Firmware
 */
#define M2M_FIRMWARE_FLASH_SZ               (236*1024UL)
/**
 *
 * OTA image Size
 */
#define OTA_IMAGE_SIZE                      (M2M_FIRMWARE_FLASH_SZ)
/**
 *
 * Flash Total size
 */
#define FLASH_IMAGE1_CONTENT_SZ             (M2M_COMMON_DATA_SEC  +  OTA_IMAGE_SIZE)

/**
 *
 * OTA image 2 offset
 */
#define M2M_OTA_IMAGE2_OFFSET               (FLASH_IMAGE1_CONTENT_SZ)

/*
 * App(Cortus App 4M): App. which runs over firmware
 *
 */
#define M2M_APP_4M_MEM_FLASH_SZ             (FLASH_SECTOR_SZ * 10)
#define M2M_APP_4M_MEM_FLASH_OFFSET         (FLASH_4M_TOTAL_SZ - M2M_APP_4M_MEM_FLASH_SZ)

/* Check if total size of content
 *  don't exceed total size of memory allowed
 **/
#if (M2M_COMMON_DATA_SEC  +  (OTA_IMAGE_SIZE *2)> FLASH_4M_TOTAL_SZ)
#error "Excced 4M Flash Size"
#endif /* (FLASH_CONTENT_SZ > FLASH_TOTAL_SZ) */

//==============================================================================
// CONSTANTS
//==============================================================================
#define SSL_MAX_OPT_LEN                 M2M_HOSTNAME_MAX_SIZE
#define SOCKET_CMD_INVALID              0x00                    // Invalid Socket command
#define SOCKET_CMD_BIND                 0x41                    // Socket Bind command
#define SOCKET_CMD_LISTEN               0x42                    // Socket Listen command
#define SOCKET_CMD_ACCEPT               0x43                    // Socket Accept command
#define SOCKET_CMD_CONNECT              0x44                    // Socket Connect command
#define SOCKET_CMD_SEND                 0x45                    // Socket Send command
#define SOCKET_CMD_RECV                 0x46                    // Socket Receive command
#define SOCKET_CMD_SENDTO               0x47                    // Socket SendTo command
#define SOCKET_CMD_RECVFROM             0x48                    // Socket RecvFrom command
#define SOCKET_CMD_CLOSE                0x49                    // Socket Close command
#define SOCKET_CMD_DNS_RESOLVE          0x4A                    // Socket DNS Resolve command
#define SOCKET_CMD_SSL_CONNECT          0x4B                    // SSL-Socket Connect command
#define SOCKET_CMD_SSL_SEND             0x4C                    // SSL-Socket Send command
#define SOCKET_CMD_SSL_RECV             0x4D                    // SSL-Socket Recieve command
#define SOCKET_CMD_SSL_CLOSE            0x4E                    // SSL-Socket Close command
#define SOCKET_CMD_SET_SOCKET_OPTION    0x4F                    // Socket Set Option command
#define SOCKET_CMD_SSL_CREATE           0x50                    // SSL-Socket Create command
#define SOCKET_CMD_SSL_SET_SOCK_OPT     0x51                    // SSL-Socket Set Option command
#define SOCKET_CMD_PING                 0x52                    // Socket Ping command

#define SOCKET_CMD_SSL_SET_CS_LIST      0x53                    // Cipher Suite command
                                                                // Recommend instead using M2M_SSL_REQ_SET_CS_LIST and
                                                                // associated response M2M_SSL_RESP_SET_CS_LIST
#define SOCKET_CMD_SSL_BIND             0x54
#define SOCKET_CMD_SSL_EXP_CHECK        0x55



//-------------------------------
// General Purpose Socket Options
//-------------------------------
#define SO_SET_UDP_SEND_CALLBACK    0   // used to enable/disable the UDP send/sendto event
#define IP_ADD_MEMBERSHIP           1   // used for enabling the sending or receiving multicast IP address
#define IP_DROP_MEMBERSHIP          2   // used for disabling the sending or receiving multicast IP address

//-------------------
// TLS Socket Options
//-------------------
// Allow an opened SSL socket to bypass the X509 certificate verification process.
// It is highly required NOT to use this socket option in production software applications.
// It is supported for debugging and testing purposes.  The option value should be cast
// to int type and it is handled as a boolean flag.
#define SO_SSL_BYPASS_X509_VERIF                            0x01

// Set the Server Name Indicator (SNI) for an SSL socket. The SNI is a NULL-terminated
// string containing the server name assocated with the connection. It must not exceed
// the size of M2M_HOSTNAME_MAX_SIZE.
#define SO_SSL_SNI                                          0x02

// This option allow the TLS to cache the session information for fast
// TLS session establishment in future connections using the TLS Protocol session
// resume features.
#define SO_SSL_ENABLE_SESSION_CACHING                       0x03

// Enable SNI validation against the server's certificate subject
// common name. If there is no SNI provided (via the SO_SSL_SNI
// option), setting this option does nothing.
#define SO_SSL_ENABLE_SNI_VALIDATION                        0x04


// SSLCipherSuiteID TLS Cipher Suite IDs
// The following list of macros defined the list of supported TLS Cipher suites.
// Each MACRO defines a single Cipher suite.
#define SSL_CIPHER_RSA_WITH_AES_128_CBC_SHA                 NBIT0
#define SSL_CIPHER_RSA_WITH_AES_128_CBC_SHA256              NBIT1
#define SSL_CIPHER_DHE_RSA_WITH_AES_128_CBC_SHA             NBIT2
#define SSL_CIPHER_DHE_RSA_WITH_AES_128_CBC_SHA256          NBIT3
#define SSL_CIPHER_RSA_WITH_AES_128_GCM_SHA256              NBIT4
#define SSL_CIPHER_DHE_RSA_WITH_AES_128_GCM_SHA256          NBIT5
#define SSL_CIPHER_RSA_WITH_AES_256_CBC_SHA                 NBIT6
#define SSL_CIPHER_RSA_WITH_AES_256_CBC_SHA256              NBIT7
#define SSL_CIPHER_DHE_RSA_WITH_AES_256_128_CBC_SHA         NBIT8
#define SSL_CIPHER_DHE_RSA_WITH_AES_256_CBC_SHA256          NBIT9
#define SSL_CIPHER_ECDHE_RSA_WITH_AES_128_CBC_SHA           NBIT10
#define SSL_CIPHER_ECDHE_RSA_WITH_AES_256_CBC_SHA           NBIT11
#define SSL_CIPHER_ECDHE_RSA_WITH_AES_128_CBC_SHA256        NBIT12
#define SSL_CIPHER_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256      NBIT13
#define SSL_CIPHER_ECDHE_RSA_WITH_AES_128_GCM_SHA256        NBIT14
#define SSL_CIPHER_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256      NBIT15

/* All ciphers that use ECC crypto only. This excludes ciphers that use RSA. They use ECDSA instead.
   These ciphers are turned off by default at startup.
   The application may enable them if it has an ECC math engine (like ATECC508). */
#define SSL_ECC_ONLY_CIPHERS                            \
(                                                       \
    SSL_CIPHER_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256  |   \
    SSL_CIPHER_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256      \
)

/* All supported ECC Ciphers including those ciphers that depend on RSA and ECC.
   These ciphers are turned off by default at startup.
   The application may enable them if it has an ECC math engine (like ATECC508). */
#define SSL_ECC_CIPHERS_ALL_128                         \
(                                                       \
    SSL_CIPHER_ECDHE_RSA_WITH_AES_128_CBC_SHA       |   \
    SSL_CIPHER_ECDHE_RSA_WITH_AES_128_CBC_SHA256    |   \
    SSL_CIPHER_ECDHE_RSA_WITH_AES_128_GCM_SHA256    |   \
    SSL_CIPHER_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256  |   \
    SSL_CIPHER_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256      \
)

#define SSL_NON_ECC_CIPHERS_AES_128                     \
(                                                       \
    SSL_CIPHER_RSA_WITH_AES_128_CBC_SHA           |     \
    SSL_CIPHER_RSA_WITH_AES_128_CBC_SHA256        |     \
    SSL_CIPHER_DHE_RSA_WITH_AES_128_CBC_SHA       |     \
    SSL_CIPHER_DHE_RSA_WITH_AES_128_CBC_SHA256    |     \
    SSL_CIPHER_RSA_WITH_AES_128_GCM_SHA256        |     \
    SSL_CIPHER_DHE_RSA_WITH_AES_128_GCM_SHA256          \
)
/*!<
    All supported AES-128 Ciphers (ECC ciphers are not counted). This is the default active group after startup.
*/


#define SSL_ECC_CIPHERS_AES_256                 \
(                                               \
    SSL_CIPHER_ECDHE_RSA_WITH_AES_256_CBC_SHA   \
)
/*!<
    ECC AES-256 supported ciphers.
*/


#define SSL_NON_ECC_CIPHERS_AES_256                     \
(                                                       \
    SSL_CIPHER_RSA_WITH_AES_256_CBC_SHA         |       \
    SSL_CIPHER_RSA_WITH_AES_256_CBC_SHA256      |       \
    SSL_CIPHER_DHE_RSA_WITH_AES_256_128_CBC_SHA |       \
    SSL_CIPHER_DHE_RSA_WITH_AES_256_CBC_SHA256          \
)
/*!<
    AES-256 Ciphers.
    This group is disabled by default at startup because the WINC1500 HW Accelerator
    supports only AES-128. If the application needs to force AES-256 cipher support,
    it could enable them (or any of them) explicitly by calling sslSetActiveCipherSuites.
*/


#define SSL_CIPHER_ALL                                  \
(                                                       \
    SSL_CIPHER_RSA_WITH_AES_128_CBC_SHA             |   \
    SSL_CIPHER_RSA_WITH_AES_128_CBC_SHA256          |   \
    SSL_CIPHER_DHE_RSA_WITH_AES_128_CBC_SHA         |   \
    SSL_CIPHER_DHE_RSA_WITH_AES_128_CBC_SHA256      |   \
    SSL_CIPHER_RSA_WITH_AES_128_GCM_SHA256          |   \
    SSL_CIPHER_DHE_RSA_WITH_AES_128_GCM_SHA256      |   \
    SSL_CIPHER_RSA_WITH_AES_256_CBC_SHA             |   \
    SSL_CIPHER_RSA_WITH_AES_256_CBC_SHA256          |   \
    SSL_CIPHER_DHE_RSA_WITH_AES_256_128_CBC_SHA     |   \
    SSL_CIPHER_DHE_RSA_WITH_AES_256_CBC_SHA256      |   \
    SSL_CIPHER_ECDHE_RSA_WITH_AES_128_CBC_SHA       |   \
    SSL_CIPHER_ECDHE_RSA_WITH_AES_128_CBC_SHA256    |   \
    SSL_CIPHER_ECDHE_RSA_WITH_AES_128_GCM_SHA256    |   \
    SSL_CIPHER_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256  |   \
    SSL_CIPHER_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256  |   \
    SSL_CIPHER_ECDHE_RSA_WITH_AES_256_CBC_SHA           \
)

//============================================================================
// CONSTANTS
//============================================================================
#define M2M_HIF_MAX_PACKET_SIZE      (1600 - 4)
/*!< Maximum size of the buffer could be transferred between Host and Firmware.
*/

#define M2M_HIF_HDR_OFFSET (sizeof(t_hifHdr) + 4)

// OTA return status
typedef enum
{
    OTA_STATUS_SUCCESS              = 0,        // OTA Success with not errors
    OTA_STATUS_FAIL                 = 1,        // OTA generic fail
    OTA_STATUS_INVALID_ARG          = 2,        // Invalid or malformed download URL
    OTA_STATUS_INVALID_RB_IMAGE     = 3,        // Invalid rollback image
    OTA_STATUS_INVALID_FLASH_SIZE   = 4,        // Flash size on device is not enough for OTA
    OTA_STATUS_AlREADY_ENABLED      = 5,        // An OTA operation is already enabled
    OTA_STATUS_UPDATE_IN_PROGRESS   = 6,        // An OTA operation update is in progress
    OTA_STATUS_IMAGE_VERIFY_FAILED  = 7,        // OTA Verification failed
    OTA_STATUS_CONNECTION_ERROR     = 8,        // OTA connection error
    OTA_STATUS_SERVER_ERROR         = 9,        // OTA server Error (file not found or else ...)
    OTA_STATUS_ABORTED              = 10
} t_m2mOtaUpdateStatusCode;


// OTA update Status type
typedef enum
{
    DL_STATUS = 1,            // OTA download file status
    SW_STATUS = 2,            // Switching to the upgrade firmware status
    RB_STATUS = 3,            // Roll-back status
    AB_STATUS         = 4             // Abort status
} tenuOtaUpdateStatusType;

// OTA Events.  See m2m_ota_handle_events().
typedef enum
{
    M2M_OTA_STATUS_EVENT = 1,
} t_m2mOtaEventType;


typedef enum
{
    M2M_SUCCESS                        = 0,
    M2M_WIFI_INVALID_CHIP_REV_ERROR    = 1,     // Read incorrect WINC1500 revision.  Either invalid WINC1500 revision or SPI interface has an error.
    M2M_WIFI_SET_CUST_INFO_ERROR       = 2,     // NULL pointer parameter in m2m_wifi_set_cust_InfoElement()
    M2M_WIFI_SET_CUST_INFO_LEN_ERROR   = 3,     // Custom info element is too large in m2m_wifi_set_cust_InfoElement()
    M2M_WIFI_SCAN_OPTIONS_ERROR        = 4,     // Invalid parameter(s) in m2m_wifi_set_scan_options()
    M2M_WIFI_SCAN_REGION_ERROR         = 5,     // Invalid parameter in m2m_wifi_set_scan_region()
    M2M_WIFI_SCAN_IN_PROGRESS_ERROR    = 6,     // Scan already in progress; see m2m_wifi_request_scan(), m2m_wifi_request_scan_passive()
    M2M_WIFI_SCAN_CHANNEL_ERROR        = 7,     // Invalid channel parameter in m2m_wifi_request_scan(), m2m_wifi_request_scan_passive()
    M2M_WIFI_P2P_CHANNEL_ERROR         = 8,     // Invalid WPS channel parameter in m2m_wifi_p2p()
    M2M_WIFI_AP_CONFIG_ERROR           = 9,     // Invalid parameter(s) in m2m_wifi_enable_ap() or m2m_wifi_start_provision_mode())
    M2M_WIFI_REQ_SLEEP_ERROR           = 11,    // m2m_wifi_request_sleep() not allowed unless in M2M_WIFI_PS_MANUAL mode
    M2M_WIFI_DEVICE_NAME_TO_LONG_ERROR = 12,    // Name parameter too long in m2m_wifi_set_device_name()
    M2M_WIFI_PROVISION_MODE_ERROR      = 13,    // Null pointer parameter in m2m_wifi_start_provision_mode()
    M2M_WIFI_CONNECT_ERROR             = 14,    // Invalid parameter(s) in m2m_wifi_connect()
    M2M_WIFI_FIRMWARE_READ_ERROR       = 15,    // Firmware read issue, read all 0's in nm_get_firmware_info()
    M2M_WIFI_FIRMWARE_MISMATCH_ERROR   = 16,    // Firmware revision mismatch in nm_get_firmware_info()
    M2M_WIFI_FIRMWARE_VERS_ZERO_ERROR  = 17,    // Read firmware version of 0 (invalid); see nm_get_firmware_full_info()
    M2M_WIFI_FIRMWARE_REG_READ_2_ERROR = 18,    // Firmware register read fail
    M2M_WIFI_PRNG_GET_ERROR            = 19,    // Invalid inputs to m2m_wifi_prng_get_random_bytes()
    M2M_WIFI_BOOTROM_LOAD_FAIL_ERROR   = 20,    // Failed to load bootrom from flash, see WaitForBootRom()
    M2M_WIFI_FIRMWARE_START_ERROR      = 21,    // Timed out waiting for WINC1500 firmware to start; see WaitForFirmwareStart()
    M2M_WIFI_WAKEUP_FAILED_ERROR       = 22,    // Wakeup failed, see ChipWake()
    M2M_WIFI_FALSE_INTERRUPT_ERROR     = 23,    // Received an interrupt with no known cause; see hif_isr()
    M2M_WIFI_INVALID_SIZE_ERROR        = 24,    // Received invalid size in hif_isr()
    M2M_WIFI_INVALID_GROUP_ERROR       = 25,    // Received invalid group in hif_isr()
    M2M_WIFI_INVALID_PACKET_SIZE_ERROR = 26,    // Received invalid packet size in hif_isr()
    M2M_WIFI_CHIP_REV_ERROR            = 27,    // Only support rev 3A0 or greater; see WaitForBootRom()
    M2M_WIFI_INVALID_WIFI_EVENT_ERROR  = 28,    // Invalid internal WiFi event, see WifiInternalEventHandler()
    M2M_WIFI_EVENT_READ_ERROR          = 30,    // Error reading event data; see WifiInternalEventHandler()
    M2M_WIFI_FAILED_TO_WAKE_CHIP_ERROR = 31,    // Failed to wake chip; see ChipWake()

    // internal errors
    M2M_WIFI_HIF_RECEIVE_1_ERROR        = 256,
    M2M_WIFI_HIF_RECEIVE_2_ERROR        = 257,
    M2M_WIFI_HIF_RECEIVE_3_ERROR        = 258,
    M2M_WIFI_HIF_RECEIVE_4_ERROR        = 259,
    M2M_WIFI_INVALID_OTA_RESPONSE_ERROR = 260,
    M2M_WIFI_MISMATCH_SESSION_ID_ERROR  = 261,    // mismatched session ID on socket send
    M2M_WIFI_FLASH_WRITE_1_ERROR        = 262,
    M2M_WIFI_FLASH_WRITE_2_ERROR        = 263,
    M2M_WIFI_FLASH_WRITE_3_ERROR        = 264,
    M2M_WIFI_FLASH_READ_ERROR           = 265,
} t_m2mWifiErrorCodes;



typedef enum
{
    REQ_GROUP_MAIN = 0,
    M2M_REQ_GROUP_WIFI,
    REQ_GROUP_IP,
    REQ_GROUP_HIF,
    REQ_GROUP_OTA,
    REQ_GROUP_SSL,
    REQ_GROUP_CRYPTO,
    REQ_GROUP_SIGMA
} t_group;

// Soft AP mode commands
typedef enum
{
    AP_REQ_ENABLE_AP = AP_CMD_BASE,         // Enable AP mode
    AP_REQ_DISABLE_AP,                      // Disable AP mode
    AP_REQ_RESTART_AP,
    AP_MAX_AP_ALL
} t_softApCmd;

// Host commands to configure WINC module
typedef enum
{
    CFG_REQ_RESTART = CONFIG_CMD_BASE,      // Restart the WINC MAC layer, it's doesn't restart the IP layer
    CFG_REQ_SET_MAC_ADDRESS,                // Set the WINC mac address (not possible for production effused boards).
    CFG_REQ_CURRENT_RSSI,                   // Request the current connected AP RSSI
    CFG_RSSI_EVENT,                         // Response to CFG_REQ_CURRENT_RSSI with the RSSI value
    CFG_REQ_GET_CONN_INFO,                  // Request connection information command
    CFG_CONN_INFO_RESPONSE_EVENT,           // Response to CFG_REQ_GET_CONN_INFO
    M2M_WIFI_REQ_SET_DEVICE_NAME,           // Set the WINC device name
    CFG_REQ_START_PROVISION_MODE,           // Start the provisioning mode for the M2M Device
    CFG_PROVISION_INFO_EVENT,               // Response to CFG_REQ_START_PROVISION_MODE
    CFG_REQ_STOP_PROVISION_MODE,            // Stop the current running provision mode
    CFG_REQ_SET_SYS_TIME,                   // Set time/date
    CFG_REQ_ENABLE_SNTP_CLIENT,             // Enable SNTP client to get the timer from SNTP server.
    CFG_REQ_DISABLE_SNTP_CLIENT,            // Disable SNTP client
    CFG_RESP_MEMORY_RECOVER,                // Reserved for debugging
    CFG_REQ_CUST_INFO_ELEMENT,              // Add Custom ELement to Beacon Managament Frame
    CFG_REQ_SCAN,                           // Request scan command
    CFG_SCAN_DONE_EVENT,                    // Scan complete notification response
    CFG_REQ_SCAN_RESULT,                    // Request Scan results command
    CFG_SCAN_RESULT_EVENT,                  // Response to CFG_REQ_SCAN_RESULT
    CFG_REQ_SET_SCAN_OPTION,                // Set Scan options "slot time, slot number .. etc"
    CFG_REQ_SET_SCAN_REGION,                // Set scan region
    CFG_REQ_SET_POWER_PROFILE,              // Set WINC1500 power mode
    CFG_REQ_SET_TX_POWER,                   // Set WINC1500 Tx power level
    CFG_REQ_SET_BATTERY_VOLTAGE,            // Set battery voltage
    CFG_REQ_SET_ENABLE_LOGS,
    CFG_REQ_GET_SYS_TIME,                   // Request time from WINC1500
    CFG_SYS_TIME_EVENT,                     // Response to CFG_REQ_GET_SYS_TIME
    CFG_REQ_SEND_ETHERNET_PACKET,           // Send Ethernet packet in bypass mode
    CFG_EVENT_ETHERNET_RX_PACKET,           // Ethernet packet received in bypass mode
    CFG_REQ_SET_MAC_MCAST,                  // Set the WINC multicast filters in bypass mode
    CFG_REQ_GET_PRNG,                       // Request random numbers
    CFG_PRNG_EVENT,                         // Response to CFG_REQ_GET_PRNG
    CFG_REQ_SCAN_SSID_LIST,                 // Request scan with list of hidden SSID plus the broadcast scan
    CFG_REQ_SET_GAINS,                      // Request set the PPA gain
    CFG_REQ_PASSIVE_SCAN,                   // Request a passive scan
    CFG_MAX_CONFIG_ALL
} t_configCmd;

// Host commands while in Station mode
typedef enum
{
    STA_REQ_CONNECT = STA_CMD_BASE,         // Connect with AP
    STA_REQ_DEFAULT_CONNECT,                // Connect with default AP
    STA_DEFAULT_CONNNECT_EVENT,             // Response to connection
    STA_REQ_DISCONNECT,                     // Request to disconnect from AP
    STA_CONN_STATE_CHANGED_EVENT,           // Connection changed response
    STA_REQ_SLEEP,                          // Set PS mode
    STA_REQ_WPS_SCAN,                       // Request WPS scan
    STA_REQ_WPS,                            // Request WPS start
    STA_REQ_START_WPS,                      // his command is for internal use by the WINC and
                                            //  should not be used by the host driver
    STA_REQ_DISABLE_WPS,                    // Disable WPS
    STA_IP_ADDRESS_ASSIGNED_EVENT,          // Response indicating that IP address was obtained
    STA_WIFI_IP_CONFIGURED_EVENT,           // This command is for internal use by the WINC and
                                            //  should not be used by the host driver
    STA_IP_CONFLICT_EVENT,                  // Response indicating a conflict in obtained IP address;
                                            //  the user should re-attempt the DHCP request
    STA_REQ_ENABLE_MONITORING,              // (not used)Enable monitor mode
    STA_REQ_DISABLE_MONITORING,             // (not used) Disable monitor mode
    STA_WIFI_WIFI_RX_PACKET_EVENT,          // (not used) Indicates a packet was received in monitor mode
    STA_REQ_SEND_WIFI_PACKET,               // (not used)Send packet in monitor mode
    STA_REQ_LSN_INTERVAL,                   // Set Wi-Fi listen interval
    STA_REQ_DOZE,                           // Force the WINC to sleep in manual PS mode
    STA_MAX_STA_ALL
} t_stationModeCmd;


// P2P commands
typedef enum
{
    P2P_REQ_P2P_INTERNAL_CONNECT = P2P_CMD_BASE,    // This command is for internal use by the WINC and
                                                    //  should not be used by the host driver
    P2P_REQ_ENABLE,                                 // Enable P2P mode
    P2P_REQ_DISABLE,                                // Disable P2P mode
    P2P_REQ_REPOST,                                 // This command is for internal use by the WINC and
                                                    //  should not be used by the host driver.
    P2P_MAX_P2P_ALL
} t_p2pCmd;

// Commands while in PS mode, not current supported.
typedef enum
{
    SERVER_REQ_CLIENT_CTRL = SERVER_CMD_BASE,
    SERVER_RESP_CLIENT_INFO,
    SERVER_REQ_SERVER_INIT,
    SERVER_MAX_SERVER_ALL
} t_serverCmd;

// OTA Commands
typedef enum
{
    OTA_REQ_NOTIF_SET_URL = M2M_OTA_CMD_BASE,
    OTA_REQ_NOTIF_CHECK_FOR_UPDATE,
    OTA_REQ_NOTIF_SCHED,
    OTA_REQ_START_FW_UPDATE,
    OTA_REQ_SWITCH_FIRMWARE,
    OTA_REQ_ROLLBACK_FW,
    OTA_RESP_NOTIF_UPDATE_INFO,
    OTA_RESP_UPDATE_STATUS,
    OTA_REQ_TEST,
    OTA_REQ_START_CRT_UPDATE,
    OTA_REQ_SWITCH_CRT_IMG,
    OTA_REQ_ROLLBACK_CRT,
    OTA_REQ_ABORT,
    OTA_MAX_ALL
} t_otaCmd;

// Crypto Commands
typedef enum
{
    CRYPTO_REQ_SHA256_INIT = M2M_CRYPTO_CMD_BASE,
    CRYPTO_RESP_SHA256_INIT,
    CRYPTO_REQ_SHA256_UPDATE,
    CRYPTO_RESP_SHA256_UPDATE,
    CRYPTO_REQ_SHA256_FINISH,
    CRYPTO_RESP_SHA256_FINISH,
    CRYPTO_REQ_RSA_SIGN_GEN,
    CRYPTO_RESP_RSA_SIGN_GEN,
    CRYPTO_REQ_RSA_SIGN_VERIFY,
    CRYPTO_RESP_RSA_SIGN_VERIFY,
    CRYPTO_MAX_ALL
} t_cryptoCmd;

typedef enum
{
    /* Request IDs corresponding to the IP GROUP. */
    IP_REQ_STATIC_IP_CONF = ((uint8_t) 10),
    IP_REQ_ENABLE_DHCP,
    IP_REQ_DISABLE_DHCP
} t_ipCmd;


typedef enum
{
    FIRMWARE_VERSION_ACTIVE = 0,    // get the firmware version currently running
    FIRMWARE_VERSION_OTA     = 1    // get the firmware version of the OTA update
} t_versionNumberType;

typedef enum
{
    REQ_CONFIG_PKT,
    REQ_DATA_PKT = 0x80 // BIT7
} t_packetType;

// SocketEvents. See m2m_socket_handle_events().
typedef enum
{
    M2M_SOCKET_BIND_EVENT          = 1,     // Bind socket event
    M2M_SOCKET_LISTEN_EVENT        = 2,     // Listen socket event
    M2M_SOCKET_DNS_RESOLVE_EVENT   = 3,     // DNS Resolution event
    M2M_SOCKET_ACCEPT_EVENT        = 4,     // Socket accept event
    M2M_SOCKET_CONNECT_EVENT       = 5,     // Socket connect event
    M2M_SOCKET_RECV_EVENT          = 6,     // Socket recv event
    M2M_SOCKET_SEND_EVENT          = 7,     // Socket send event
    M2M_SOCKET_SENDTO_EVENT        = 8,     // Socket sendto event
    M2M_SOCKET_RECVFROM_EVENT      = 9,     // Socket recvfrom event
    M2M_SOCKET_PING_RESPONSE_EVENT = 10     // Ping response event
} t_m2mSocketEventType;

typedef enum
{
    M2M_SSL_REQ_CERT_VERIF,
    M2M_SSL_REQ_ECC,
    M2M_SSL_RESP_ECC,
    M2M_SSL_IND_CRL,
    M2M_SSL_IND_CERTS_ECC,
    M2M_SSL_REQ_SET_CS_LIST,
    M2M_SSL_RESP_SET_CS_LIST
} t_m2mSslCmd;

typedef int8_t  SOCKET;


// SSL Certificate Expiry Validation Options
typedef enum
{

    SSL_CERT_EXP_CHECK_DISABLE, // ALWAYS OFF.  Ignore certificate expiration date validation.
                                // If a certificate is expired or there is no configured system time,
                                // the SSL connection SUCCEEDs.

    SSL_CERT_EXP_CHECK_ENABLE,  // ALWAYS ON.  Validate certificate expiration date.
                                // If a certificate is expired or there is no configured
                                // system time, the SSL connection FAILs.

    SSL_CERT_EXP_CHECK_EN_IF_SYS_TIME  // CONDITIONAL VALIDATION (Default setting at startup).
                                       // Validate the certificate expiration date only
                                       // if there is a configured system time.  If there
                                       // is no configured system time, the certificate
                                       // expiration is bypassed and the SSL connection SUCCEEDs.
} t_m2mSslCertExpSettings;

typedef enum
{
    M2M_PING_SUCCESS          = 0,
    M2M_PING_DEST_UNREACHABLE = 1,
    M2M_PING_TIMEOUT          = 2
} t_m2mPingErrorCode;

typedef enum
{
    SOCK_ERR_NO_ERROR               =  0,       // Successful socket operation
    SOCK_ERR_INVALID_ADDRESS        = -1,       // Socket address is invalid. The socket operation cannot
                                                //  be completed successfully without specifying a specific
                                                //  address For example: bind is called without specifying a
                                                //  port number
    SOCK_ERR_ADDR_ALREADY_IN_USE    = -2,       // Socket operation cannot bind on the given address.
                                                //  With socket operations, only one IP address per socket
                                                //  is permitted. Any attempt for a new socket to bind
                                                //  with an IP address already bound to another open
                                                //  socket will return the following error code.
    SOCK_ERR_MAX_TCP_SOCK           = -3,       // Exceeded the maximum number of TCP sockets. See TCP_SOCK_MAX.
    SOCK_ERR_MAX_UDP_SOCK           = -4,       // Exceeded the maximum number of UDP sockets.  See UDP_SOCK_MAX.
    SOCK_ERR_INVALID_ARG            = -6,       // Invalid argument is passed to a function socket()
    SOCK_ERR_MAX_LISTEN_SOCK        = -7,       // Exceeded the maximum number of TCP passive listening sockets
    SOCK_ERR_INVALID                = -9,       // The requested socket operation is not valid in the
                                                //  current socket state. For example, it would not be valid
                                                //  to call accept before calling bind or listen.
    SOCK_ERR_ADDR_IS_REQUIRED       = -11,      // Destination address is required. Failed to provide the socket
                                                //  address required for the socket operation to be completed.
                                                //  This error is generated if sendto function called when
                                                //  the address required to send the data to is not known.
    SOCK_ERR_CONN_ABORTED           = -12,      // The socket is closed by the peer. The local socket is
                                                //  closed also.
    SOCK_ERR_TIMEOUT                = -13,      // The socket pending operation has  timed out
    SOCK_ERR_BUFFER_FULL            = -14,      // No buffer space available to be used for the requested
                                                //  socket operation
} t_socketError;

typedef enum
{
    M2M_TLS_CRL_TYPE_NONE            = 0,  // No CRL check
    M2M_TLS_CRL_TYPE_CERT_HASH       = 1   // CRL contains certificate hashes
} t_m2mCrlType;


// Default Connection Error Definitions
typedef enum
{
    M2M_WIFI_DEFAULT_CONN_IN_PROGRESS = ((int8_t)-23),   // default connection or forced connection is in progress
    M2M_DEFAULT_CONN_FAIL,                          // WINC1500 failed to connect to the cached network
    M2M_DEFAULT_CONN_SCAN_MISMATCH,                 // none of cached networks found in scan result
    M2M_DEFAULT_CONN_EMPTY_LIST                     // connection list is empty
} tenuM2mDefaultConnErrcode;

#if 0
typedef enum
{
    OTA_SUCCESS                     = 0,            // OTA successful
    OTA_ERR_WORKING_IMAGE_LOAD_FAIL = ((int8_t)-1), // Failure to load the firmware image
    OTA_ERR_INVAILD_CONTROL_SEC     = ((int8_t)-2), // Control structure is being corrupted
    OTA_ERR_SWITCH_FAIL             = ((int8_t)-3), // Switching to the updated image failed as may be the image is invalid
    OTA_ERR_START_UPDATE_FAIL       = ((int8_t)-4), // OTA update fail due to multiple reasons
                                                    //   - Connection failure
                                                    //   - Image integrity fail
    OTA_ERR_ROLLBACK_FAIL           = ((int8_t)-5), // Roll-back failed due to Roll-back image is not valid
    OTA_ERR_INVAILD_FLASH_SIZE      = ((int8_t)-6), // Current FLASH is less than 4MB
    OTA_ERR_INVAILD_ARG             = ((int8_t)-7), // Invalid argument in any OTA Function
    OTA_ERR_OTA_IN_PROGRESS         = ((int8_t)-8)  // OTA still in progress
} t_otaErrorCode;
#endif

// see tstrM2mWifiStateChanged
typedef enum
{
    M2M_ERR_SCAN_FAIL = ((uint8_t)1), // Failed to perform the scan operation.
    M2M_ERR_JOIN_FAIL,                // Failed to join the BSS
    M2M_ERR_AUTH_FAIL,                // Failed to authenticate with the AP
    M2M_WIFI_ERR_ASSOC_FAIL,          // Failed to associate with the AP
    M2M_ERR_CONN_INPROGRESS           // Another connection request in progress
} tenuM2mConnChangedErrcode;

typedef enum
{
    M2M_WIFI_WEP_KEY_INDEX_1 = ((uint8_t) 1),
    M2M_WIFI_WEP_KEY_INDEX_2,
    M2M_WIFI_WEP_KEY_INDEX_3,
    M2M_WEP_KEY_INDEX_4
} tenuM2mWepKeyIndex;

typedef enum
{
    PWR_AUTO = ((uint8_t) 1),   // FW will decide the best power mode to use internally
    PWR_LOW1,                   // low power mode #1
    PWR_LOW2,                   // low power mode #2
    PWR_HIGH                    // high power mode
} tenuM2mPwrMode;

typedef enum
{
    TX_PWR_HIGH = ((uint8_t) 1),     // PPA Gain 6dbm    PA Gain 18dbm
    TX_PWR_MED,                      // PPA Gain 6dbm    PA Gain 12dbm
    TX_PWR_LOW                       // PPA Gain 6dbm    PA Gain 6dbm
} tenuM2mTxPwrLevel;

typedef struct
{
    //Note: on SAMD D21 the size of double is 8 Bytes
    uint16_t   u16BattVolt;
    uint8_t    padding[2];
} tstrM2mBatteryVoltage;

typedef enum
{
    M2M_WIFI_DISCONNECTED = 0,       // Wi-Fi state is disconnected
    M2M_WIFI_CONNECTED,              // Wi-Fi state is connected
    M2M_WIFI_UNDEF = 0xff            // Undefined Wi-Fi State
} tenuM2mConnState;

typedef enum
{
    M2M_WIFI_SEC_INVALID = 0,        // Invalid security type
    M2M_WIFI_SEC_OPEN,               // open security
    M2M_WIFI_SEC_WPA_PSK,            // WPA/WPA2 personal(PSK)
    M2M_WIFI_SEC_WEP,                // WEP (40 or 104) OPEN OR SHARED
    M2M_WIFI_SEC_802_1X              // WPA/WPA2 Enterprise.IEEE802.1x user-name/password authentication
} tenuM2mSecType;

typedef enum
{
    SSID_MODE_VISIBLE = 0,       // SSID is visible to others
    SSID_MODE_HIDDEN             // SSID is hidden
} tenuM2mSsidMode;

// Wi-Fi channels
typedef enum
{
    M2M_WIFI_CH_1 = ((uint8_t) 1),
    M2M_WIFI_CH_2,
    M2M_WIFI_CH_3,
    M2M_WIFI_CH_4,
    M2M_WIFI_CH_5,
    M2M_WIFI_CH_6,
    M2M_WIFI_CH_7,
    M2M_WIFI_CH_8,
    M2M_WIFI_CH_9,
    M2M_WIFI_CH_10,
    M2M_WIFI_CH_11,
    M2M_WIFI_CH_12,
    M2M_WIFI_CH_13,
    M2M_WIFI_CH_14,
    M2M_WIFI_CH_ALL = ((uint8_t) 255)
} tenuM2mScanCh;

// see m2m_wifi_set_scan_region()
typedef enum
{
    M2M_WIFI_NORTH_AMERICA_REGION = ((uint16_t) 0x7FF),     // 11 channels
    M2M_WIFI_EUROPE_REGION        = ((uint16_t) 0x1FFF),    // 13 channels
    M2M_WIFI_NORTH_ASIA_REGION    = ((uint16_t) 0x3FFF)     // 14 channels
} tenuM2mScanRegion;

// Power Save Modes
typedef enum
{
    M2M_NO_PS,   // Power save is disabled.

    M2M_WIFI_PS_AUTOMATIC,  // Power save is done automatically by the WINC1500.
                            // This mode doesn't disable all of the WINC modules and
                            //  use higher levels of power than the M2M_WIFI_PS_H_AUTOMATIC and
                            //  the M2M_WIFI_PS_DEEP_AUTOMATIC modes

    M2M_WIFI_PS_H_AUTOMATIC,// Power save is done automatically by the WINC1500.
                            //  Achieves higher power save than the M2M_WIFI_PS_AUTOMATIC mode
                            //  by shutting down more parts of the WINC board

    M2M_WIFI_PS_DEEP_AUTOMATIC,   // Power save is done automatically by the WINC1500.
                                  //  Achieves the highest possible power save.

    M2M_WIFI_PS_MANUAL            // Power save is done manually by host application
} tenuPowerSaveModes;

typedef enum
{
    M2M_WIFI_MODE_NORMAL = ((uint8_t) 1),   // run customer firmware version
    M2M_WIFI_MODE_ATE_HIGH,                 // Config Mode in HIGH POWER means to run production test firmware version which is known as ATE (Burst) firmware
    M2M_WIFI_MODE_ATE_LOW,                  // Config Mode in LOW POWER means to run production test firmware version which is known as ATE (Burst) firmware
    M2M_WIFI_MODE_ETHERNET,                 // Ethernet mode
    M2M_WIFI_MODE_MAX
} tenuM2mWifiMode;

typedef enum
{
    WPS_PIN_TRIGGER = 0,                // WPS is triggered in PIN method
    WPS_PBC_TRIGGER = 4                 // WPS is triggered via push button
} tenuWPSTrigger;

























typedef struct
{
    uint32_t macEfuseMib;
    uint32_t firmwareOtaRev;
} t_gpRegs;


#if 0
// OTA Image Header
typedef struct
{
    uint32_t u32OtaMagicValue;     // Magic value kept in the OTA image after the
                                //  sha256 Digest buffer to define the Start of OTA Header
    uint32_t otaPayloadSzie;    // Total OTA image payload size, include the sha256 key size
} t_otaInitHeader;
#endif

// Control section structure is used to define the working image and
// the validity of the roll-back image and its offset, also both firmware versions is
// kept in that structure.
typedef struct
{
    uint32_t u32OtaMagicValue;         // Magic value used to ensure the structure is valid or not
    uint32_t u32OtaFormatVersion;      // NA   NA   NA   Flash version   cs struct version
                                    // 00   00   00   00              00
                                    // Control structure format version, the value will be incremented in
                                    // case of structure changed or updated
    uint32_t u32OtaSequenceNumber;     // Sequence number is used while update the control structure to keep track of how many times that section updated
    uint32_t u32OtaLastCheckTime;      // Last time OTA check for update
    uint32_t u32OtaCurrentworkingImagOffset;       // Current working offset in flash
    uint32_t u32OtaCurrentworkingImagFirmwareVer;  // current working image version ex 18.0.1
    uint32_t u32OtaRollbackImageOffset;            // Roll-back image offset in flash
    uint32_t u32OtaRollbackImageValidStatus;       // roll-back image valid status
    uint32_t u32OtaRollbackImagFirmwareVer;        // Roll-back image version (ex 18.0.3)
    uint32_t u32OtaCortusAppWorkingOffset;         // cortus app working offset in flash
    uint32_t u32OtaCortusAppWorkingValidSts;       // Working Cortus app valid status
    uint32_t u32OtaCortusAppWorkingVer;            // Working cortus app version (ex 18.0.3)
    uint32_t u32OtaCortusAppRollbackOffset;        //cortus app rollback offset in flash
    uint32_t u32OtaCortusAppRollbackValidSts;      // roll-back cortus app valid status
    uint32_t u32OtaCortusAppRollbackVer;           // Roll-back cortus app version (ex 18.0.3)
    uint32_t u32OtaControlSecCrc;                  // CRC for the control structure to ensure validity
} tstrOtaControlSec;

typedef struct
{
    uint8_t    u8PwrMode;  // power Save Mode (see tenuM2mPwrMode)
    uint8_t    padding[3];
} tstrM2mPwrMode;

typedef struct
{
    uint8_t    u8TxPwrLevel; // see tenuM2mTxPwrLevel
    uint8_t    padding[3];
} tstrM2mTxPwrLevel;

typedef struct
{
    uint8_t    u8Enable;      // Enable/Disable firmware logs
    uint8_t   padding[3];
} tstrM2mEnableLogs;

#define CONNECTION_INFO_PAD_SIZE    (4 - ((sizeof(tstrM2MWifiSecInfo) + M2M_MAX_SSID_LEN + 3) % 4))
typedef struct
{
    tstrM2MWifiSecInfo    securityInfo;           // Security parameters for authenticating with the AP
    uint16_t              channel;                // RF Channel for the target SSID
    uint8_t               ssid[M2M_MAX_SSID_LEN];  // SSID of the desired AP. It must be NULL terminated string
    uint8_t               noSaveCred;
    uint8_t               padding[CONNECTION_INFO_PAD_SIZE];
} t_connectConfig;

typedef struct
{
    uint8_t     u8ListenChannel;          // P2P Listen Channel (1, 6 or 11)
    uint8_t     padding[3];
} tstrM2MP2PConnect;

// Sets the MAC address from application. The WINC load the mac address from the effuse by default to the WINC configuration memory,
// but that function is used to let the application overwrite the configuration memory with the mac address from the host.
// It's recommended to call this only once before calling connect request and after the m2m_wifi_init
typedef struct
{
    uint8_t  au8Mac[6];          // MAC address
    uint8_t  padding[2];
} tstrM2mSetMacAddress;

// It is assigned by the application. It is used mainly for Wi-Fi Direct device
// discovery and WPS device information.
typedef struct
{
    char  au8DeviceName[M2M_DEVICE_NAME_MAX]; // NULL-terminated device name
} tstrM2MDeviceNameConfig;

// Provisioning Mode Configuration
typedef struct
{
    tstrM2MAPConfig    strApConfig;                  // Configuration parameters for the WiFi AP
    char               acHttpServerDomainName[64];   // The device domain name for HTTP provisioning
    uint8_t            u8EnableRedirect;             // A flag to enable/disable HTTP redirect feature for
                                                     // the HTTP Provisioning server. If the Redirect is enabled,
                                                     // all HTTP traffic (http://URL) from the device associated
                                                     // with WINC AP will be redirected to the HTTP Provisioning
                                                     // Web page.
                                                     //   - 0 : Disable HTTP Redirect
                                                     //   - 1 : Enable HTTP Redirect.
    uint8_t         padding[3];
} tstrM2MProvisionModeConfig;

/**
*    @struct        t_hifHdr
*    @brief        Structure to hold HIF header
*/
typedef struct
{
    uint8_t   groupId; // group ID
    uint8_t   opCode;  // op code
    uint16_t  length;  // payload length
} t_hifHdr;


// This structure needs to be 8 bytes.  If a pointer is less then 4 bytes then
// need some extra padding.
typedef struct
{
    uint8_t     *p_buf;                        // return buffer
#if ( M2M_POINTER_SIZE_IN_BYTES == 2 )
    uint8_t     extraPadding[2];
#elif ( M2M_POINTER_SIZE_IN_BYTES == 3 )      // PIC18
    uint8_t     extraPadding[1];
#endif
    uint16_t    size;           // PRNG size requested
    uint8_t     padding[2];     // not used
} t_prng;


// Update Information
typedef struct
{
    uint8_t     u8OtaUpdateStatusType;  // see tenuOtaUpdateStatusType
    uint8_t     u8OtaUpdateStatus;      // see t_m2mOtaUpdateStatusCode
    uint8_t     padding[2];
} tstrOtaUpdateStatusResp;


// Pointer to this union is passed to m2m_ota_handle_events()
typedef struct t_m2mOtaEventData
{
    tstrOtaUpdateStatusResp  otaUpdateStatus;
} t_m2mOtaEventData;

// TLS certificate revocation list.  Certificate data for inclusion in a revocation list (CRL)
typedef struct
{
    uint8_t    dataLen;                    // Length of certificate data (maximum is M2M_WIFI_TLS_CRL_DATA_MAX_LEN)
    uint8_t    data[M2M_WIFI_TLS_CRL_DATA_MAX_LEN]; // certificate data
    uint8_t    padding[3];                 // do not use
} t_m2mWifiTlsCrlEntry;

// Certificate revocation list details
typedef struct
{
    uint8_t            crlType;                     // Type of certificate data contained in list
                                                    // M2M_TLS_CRL_TYPE_NONE or TLS_CRL_TYPE_HASH
    uint8_t            padding[3];                  // do not use
    t_m2mWifiTlsCrlEntry    tlsCrl[M2M_WIFI_TLS_CRL_MAX_ENTRIES]; // list of CRL's
} tstrTlsCrlInfo;

// Pointers to socket address structures are typically cast to pointers of this type before
// use in socket function calls
struct sockaddr
{
    uint16_t    sa_family;      // Socket address family (always AF_INET)
    uint8_t     sa_data[14];    // Maximum size of all the different socket address structures
};

// Used in other socket structures
typedef struct
{
    uint32_t            s_addr;  // Network Byte Order representation of the IPv4 address. For example,
                                 //  the address "192.168.0.10" is represented as 0x0A00A8C0.
} in_addr;

// Socket address structure for IPV4 addresses. Used to specify socket address information
// to which to connect to.  Can be cast to sockaddr structure.
struct sockaddr_in
{
    uint16_t    sin_family;     // Must be AF_INET (only ipV4 supported)
    uint16_t    sin_port;       // Port number of socket.  Network sockets are
                                //  identified by a pair of IP addresses and port number.
                                //  It must be set in the Network Byte Order format.
                                //  Cannot have a zero value.

    in_addr     sin_addr;       // IP Address of the socket.  See in_addr structure.
                                //  Can be set to 0 to accept any IP address for server
                                //  option.  Otherwise must be non-zero.

    uint8_t     padding[8];     // Padding to make structure the same size as sockaddr struct
};


// One of the event structures in t_socketEventData used when a remote host has
// accepted a connection.  Corresponds to the M2M_SOCKET_ACCEPT_EVENT.
typedef struct
{
    SOCKET              sock;    // On a successful accept operation, the return information
                                 //  is the socket ID for the accepted connection with the
                                 //  remote peer. Otherwise a negative error code is returned
                                 //  to indicate failure of the accept operation.

    struct sockaddr_in  strAddr; //Socket address structure for the remote peer.
} t_socketAccept;

// One of the event structures in t_socketEventData used when a connection has
// occurred.  Corresponds to the M2M_SOCKET_CONNECT_EVENT.
typedef struct
{
    SOCKET    sock;        // Socket ID referring to the socket passed to the connect function call
    int8_t    error;       // Connect error code.  Zero if successful, else negative.
                           //  See t_socketError
} t_socketConnect;

// One of the event structures in t_socketEventData used when a DNS reply is received.
// Corresponds to the M2M_SOCKET_DNS_RESOLVE_EVENT.
typedef struct
{
    char        hostName[M2M_HOSTNAME_MAX_SIZE]; // host name string
    uint32_t    hostIp;                         // host IP address (big-endian)
} t_dnsReply;

// One of the event structures in t_socketEventData used when socket data is received.
// Corresponds to the M2M_SOCKET_RECV_EVENT or M2M_SOCKET_RECVFROM_EVENT events.
// In case the received data from the remote peer is larger than the user buffer size defined
// during the asynchronous call to the recv function, the data is delivered to the user in a
// number of consecutive chunks according to the user Buffer size.  A negative or zero buffer
// size indicates an error with the following code:
//    SOCK_ERR_NO_ERROR      - Socket connection  closed
//    SOCK_ERR_CONN_ABORTED  - Socket connection aborted
//    SOCK_ERR_TIMEOUT       - Socket receive timed out
typedef struct
{
    uint8_t     *p_rxBuf;       // Pointer to the user buffer (passed to recv or recvfrom function)
                                //  containing the received data chunk

    int16_t     bufSize;        // The received data chunk size.  Holds a negative value
                                //  if there is a receive error or ZERO on success upon reception
                                //  of close socket message.

    uint16_t    remainingSize;  // The number of bytes remaining in the current recv operation

    struct sockaddr_in  ai_addr; // Only valid for M2M_SOCKET_RECVFROM_EVENT event (UDP sockets).
                                 // Socket address structure for the remote peer.
} t_socketRecv;

// One of the event structures in t_socketEventData used when a Ping response
// is received.  Corresponds to the M2M_SOCKET_PING_RESPONSE_EVENT.
typedef struct
{
    uint32_t              u32StaticIp;   // IP address of ping responder
    uint32_t              rtt;           // round trip time of ping
    t_m2mPingErrorCode    errorCode;     // M2M_PING_SUCCESS if successful
} t_pingReply;


typedef union t_socketEventData
{
    int8_t            bindStatus;             // 0 if bind successful, else error code
    int8_t            listenStatus;           // 0 if listen successful, else error code
    int16_t           numSendBytes;
    t_socketAccept    acceptResponse;
    t_socketConnect   connectResponse;
    t_dnsReply        dnsReply;
    t_socketRecv      recvMsg;
    t_pingReply       pingReply;
} t_socketEventData;

typedef struct
{
    uint8_t     *p_userBuf;
    uint16_t    userBufSize;
    uint16_t    sessionId;
    uint16_t    dataOffset;
    uint8_t     isUsed;
    uint8_t     sslFlags;
    uint8_t     isRecvPending;
} t_socket;

typedef struct
{
    uint16_t    family;
    uint16_t    port;
    uint32_t    ipAddr;
} t_sockAddr;

typedef struct
{
    t_sockAddr  addr;
    SOCKET      sock;
    uint8_t     notUsed;
    uint16_t    sessionId;
} t_bindCmd;

typedef struct
{
    SOCKET      sock;       // socket ID
    int8_t      status;     // bind status (SOCK_ERR_NO_ERROR for success)
    uint16_t    sessionId;
} t_bindReply;

typedef struct
{
    SOCKET      sock;
    uint8_t     backlog;
    uint16_t    sessionId;
} t_listenCmd;

typedef struct
{
    SOCKET      sock;
    int8_t      status;
    uint16_t    sessionId;
} t_listenReply;

typedef struct
{
    t_sockAddr  addr;
    SOCKET      listenSock;
    SOCKET      connectSock;
    uint16_t    appDataOffset; // In further packet send requests the host interface should put the user application
                               //  data at this offset in the allocated shared data packet.
} t_acceptReply;


typedef struct
{
    t_sockAddr  addr;
    SOCKET      sock;
    uint8_t     sslFlags;
    uint16_t    sessionId;
} t_connectCmd;

typedef struct
{
    SOCKET      sock;
    uint8_t     notUsed;
    uint16_t    sessionId;
} t_closeCmd;

typedef struct
{
    SOCKET      sock;
    int8_t      error;
    uint16_t    appDataOffset;  // In further packet send requests the host interface
                                //  should put the user application
} t_socketConnectReply;

typedef struct
{
    SOCKET      sock;
    uint8_t     notUsed1;
    uint16_t    dataSize;
    t_sockAddr  addr;
    uint16_t    sessionId;
    uint16_t    notUsed2;
} t_sendCmd;

typedef struct
{
    SOCKET      sock;
    uint8_t     notUsed1;
    int16_t     sentBytes;
    uint16_t    sessionId;
    uint16_t    notUsed2;
} t_sendReply;

typedef struct
{
    uint32_t    timeout;
    SOCKET      sock;
    uint8_t     notUsed1;
    uint16_t    sessionId;
} t_recvCmd;

typedef struct
{
    t_sockAddr  addr;
    int16_t     recvStatus;
    uint16_t    dataOffset;
    SOCKET      sock;
    uint8_t     notUsed;
    uint16_t    sessionId;
} t_recvReply;

typedef struct
{
    uint32_t    optionValue;
    SOCKET      sock;
    uint8_t     option;
    uint16_t    sessionId;
} t_setSockOptCmd;


typedef struct
{
    SOCKET      sslSock;
    uint8_t     padding[3];
} t_sslSocketCreateCmd;

typedef struct
{
    SOCKET      sock;
    uint8_t     option;
    uint16_t    sessionId;
    uint32_t    optLength;
    uint8_t     optValue[SSL_MAX_OPT_LEN];
} t_sslSetSockOptCmd;

typedef struct
{
    uint32_t    destIpAddress;
    uint32_t    reserved;
    uint16_t    pingCount;
    uint8_t     ttl;
    uint8_t     padding;
} tstrPingCmd;

typedef struct
{
    uint32_t    cipherSuiteMask;
} t_sslActiveCipherSuites;

typedef struct
{
    uint32_t    enable;
} t_sslCertExpSettings;

// WEP security parameters
typedef struct
{
    uint8_t    u8KeyIndx;        // WEP key Index
    uint8_t    u8KeySz;         // WEP key size
    uint8_t    au8WepKey[WEP_104_KEY_STRING_SIZE + 1]; // WEP Key represented as a NULL terminated ASCII string
    uint8_t    padding[3];
} tstrM2mWifiWepParams;


// Credentials for the user to authenticate with the AAA server (WPA-Enterprise Mode IEEE802.1x).
typedef struct
{
    uint8_t    au8UserName[M2M_1X_USR_NAME_MAX];      //User Name. It must be Null terminated string
    uint8_t    au8Passwd[M2M_WIFI_PASSWORD_1X_MAX];   // Password corresponding to the user name. It must be Null terminated string
} tstr1xAuthCredentials;

//  Wi-Fi Security Parameters for all supported security modes.
typedef union
{
    uint8_t                 au8PSK[M2M_MAX_PSK_LEN];  // Pre-Shared Key in case of WPA-Personal security
    tstr1xAuthCredentials   strCred1x;                // Credentials for RADIUS server authentication in case of WPA-Enterprise security
    tstrM2mWifiWepParams    strWepInfo;               // WEP key parameters in case of WEP security
} tuniM2MWifiAuth;


// Authentication credentials to connect to a Wi-Fi network
#define SECURITY_INFO_PAD_SIZE      (4 - ((sizeof(tuniM2MWifiAuth) + 1) % 4))
typedef struct
{
    tuniM2MWifiAuth    uniAuth;           // Union holding all possible authentication parameters corresponding the current security types
    uint8_t            u8SecType;         // Wi-Fi network security type.  See tenuM2mSecType.
    uint8_t            padding[SECURITY_INFO_PAD_SIZE];
} tstrM2MWifiSecInfo;


// WPS Configuration parameters
typedef struct
{
    uint8_t    u8TriggerType;         // WPS_PBC_TRIGGER or WPS_PIN_TRIGGER
    char       acPinNumber[8];       // WPS Pin
    uint8_t    padding[3];
} tstrM2MWPSConnect;


// Scan config
typedef struct
{
    uint8_t   u8NumOfSlot;  // The min number of slots is 2 for every channel.  Every
                            //  slot the WINC will send a probe request and then wait/listen
                            //  for a probe response/beacon u8SlotTime

    uint8_t   u8SlotTime;   // The time (in ms) that the WINC will wait on every channel listening
                            // for AP's.  The min time is 10ms; the max time is 250ms.

    uint8_t   u8ProbesPerSlot; // Number of probe requests to be sent per channel scan slot

    int8_t    s8RssiThresh;  // RSSI threshold of the AP
} tstrM2MScanOption;

typedef struct
{
    uint16_t u16ScanRegion;           // see tstrM2MScanRegion
    uint8_t padding[2];
} tstrM2MScanRegion;

typedef struct
{
    uint8_t    u8ChNum;         // see tenuM2mScanCh
    uint8_t    padding[1];
    uint16_t   passiveScanTime;

} tstrM2MScan;


typedef struct
{
    uint8_t    u8Index;      // Index of the desired scan result
    uint8_t    padding[3];
} tstrM2mReqScanResult;

typedef struct
{
    uint8_t     u8PsType;         // see tenuPowerSaveModes
    uint8_t     u8BcastEn;       // 1: WINC1500 must be awake each DTIM Beacon for receiving Broadcast traffic
                                 // 0: WINC1500 will not wakeup at the DTIM Beacon, but its wakeup depends only
                                 //     on the the configured Listen Interval
    uint8_t     padding[2];
} tstrM2mPsType;


//  Manual power save request sleep time
typedef struct
{
    uint32_t u32SleepTime;  // sleep time in ms
} tstrM2mSlpReqTime;


// Configuration parameters for the Soft AP mode.
typedef struct
{
    uint8_t     au8SSID[M2M_MAX_SSID_LEN];      // SSID of SoftAP network
    uint8_t     u8ListenChannel;                // channel to use for SoftAP
    uint8_t     u8KeyIndx;                      // WEP key index
    uint8_t     u8KeySz;                        // M2M_WIFI_WEP_40_KEY_STRING_SIZE, M2M_WIFI_WEP_104_KEY_STRING_SIZE, or WPA key size
    uint8_t     au8WepKey[WEP_104_KEY_STRING_SIZE + 1];
    uint8_t     u8SecType;                      // M2M_WIFI_SEC_OPEN, M2M_WIFI_SEC_WEP, or M2M_WIFI_SEC_WPA_PSK
    uint8_t     u8SsidHide;                     // SSID_MODE_VISIBLE or SSID_MODE_HIDDEN
    uint8_t     au8DHCPServerIP[4];             // AP IP server address
    uint8_t     au8Key[M2M_MAX_PSK_LEN];        // WPA key, if used
    uint8_t     __PAD24__[2];
} tstrM2MAPConfig;

typedef struct
{
    uint16_t 	u16LsnInt;      // Listen interval in Beacon period count.
    uint8_t     padding[2];     // not used
} tstrM2mLsnInt;

// This struct contains a TLS certificate.
typedef struct
{
    char        acFileName[TLS_FILE_NAME_MAX];  // Name of the certificate
    uint32_t    u32FileSize;                    // Size of the certificate
    uint32_t    u32FileAddr;                    // Error Code
} tstrTlsSrvSecFileEntry;

// This struct contains a set of TLS certificates.
typedef struct
{
    uint8_t     au8SecStartPattern[TLS_SRV_SEC_START_PATTERN_LEN];   // Start pattern
    uint32_t    u32nEntries;                                         // Number of certificates stored in the struct
    uint32_t    u32NextWriteAddr;                                    // TLS Certificates
    tstrTlsSrvSecFileEntry astrEntries[TLS_SRV_SEC_MAX_FILES];
} tstrTlsSrvSecHdr;



/*!
@typedef typedef void (*tpfHifCallBack)(uint8_t opCode, uint16 dataSize, uint32 address);
@brief    used to point to Wi-Fi call back function depend on Arduino project or other projects.
@param [in]    opCode
                HIF Opcode type.
@param [in]    dataSize
                HIF data length.
@param [in]    address
                HIF address.
@param [in]    grp
                HIF group type.
*/
typedef void (*tpfHifCallBack)(uint8_t opCode, uint16_t dataSize, uint32_t address);
typedef void (*tpfAppSocketCb) (SOCKET sock, uint8_t u8Msg, void * pvMsg);
typedef void (*tpfAppResolveCb) (char* pu8DomainName, uint32_t u32ServerIP);





























#endif // WIFI_7_H
