/*******************************************************************************
  Reset System Service Interface Definition

  Company:
    Microchip Technology Inc.

  File Name:
    sys_reset.h

  Summary:
    Reset System Service interface definition.

  Description:
    This file contains the interface definition for the Reset System
    Service.  It provides a way to interact with the Reset subsystem to
    manage the system resets and their status.
*******************************************************************************/

//DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
*******************************************************************************/
//DOM-IGNORE-END

#ifndef _SYS_RESET_H
#define _SYS_RESET_H


// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include "system_config.h"

// *****************************************************************************
// *****************************************************************************
// Section: SYS RESET Module Routines
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
/* Function:
    void SYS_RESET_SoftwareReset( void )

  Summary:
    Triggers a software reset

  Description:
    This function triggers a processor reset. The function maps to the default
    implementation for each compiler. The implementation can be overridden if
    required. To use one of the default implementations for compiler, call
    SYS_RESET_SoftwareReset(), wherever required.

    system.h should have included this file.

    To override this function, in the system_config.h have:
        #define SYS_RESET_SoftwareReset() MyReset()

    Implement the function MyReset in system.c and provide a prototype in
    in system.h.

    In system.h
        void MyReset(void);

    In system.c
        void MyReset(void)
        {
            // User defined reset behavior
        }

  PreCondition:
    None.

  Parameters:
    None.
      
  Returns:
    None.

*/

#ifndef SYS_RESET_SoftwareReset
    #if defined(__XC8__)||defined(__XC16__)
        #define SYS_RESET_SoftwareReset()        asm("reset")
    #elif defined(__XC32__)
        #define SYS_RESET_SoftwareReset()        SoftReset()
    #else
        #warning "Define and/or declare SYS_RESET_SoftwareReset() in system_config.h and system.h."
    #endif
#endif

#endif //_SYS_RESET_H

/*******************************************************************************
 End of File
*/

